﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Data.Linq;
using System.Data.SqlClient;
using System.Configuration;
using LinqToSqlResearch20080310.Business;
using System.Data.Linq.Mapping;

namespace Linq2SqlResearch20080310.UnitTests
{
    /// <summary>
    /// Summary description for UnitTest1
    /// </summary>
    [TestClass]
    public class ZipCodeInfoFixture : UnitTestBase
    {
        public ZipCodeInfoFixture()
        {            
        }

        public void CreateDatabase()
        {
            if (this.Context.DatabaseExists() == true)
            {
                this.Context.DeleteDatabase();
            }

            this.Context.CreateDatabase();
        }

        private DataContext CreateDataContext()
        {
            DataContext context;

            ConnectionStringSettings connectionString = ConfigurationManager.ConnectionStrings["default"];

            if (connectionString == null)
            {
                throw new InvalidOperationException("Could not locate 'default' connection string in app.config.");
            }

            context = new DataContext(new SqlConnection(connectionString.ConnectionString), new AttributeMappingSource());
            context.Log = Console.Out;
            context.GetTable<ZipCodeInfo>();

            return context;
        }

        private DataContext m_context;
        public DataContext Context
        {
            get
            {
                if (m_context == null)
                {
                    m_context = CreateDataContext();
                }

                return m_context;
            }
            set
            {
                m_context = value;
            }
        }

        [TestInitialize]
        public void TestInitialize()
        {
            if (Context.DatabaseExists() == true)
            {
                Context.DeleteDatabase();
            }

            Context.CreateDatabase();
        }

        [TestCleanup]
        public void TestCleanup()
        {
            if (m_context != null)
            {
                m_context.Dispose();
                m_context = null;
            }
        }

        [TestMethod]
        public void Create()
        {
            ZipCodeInfo zip = new ZipCodeInfo()
                {
                    AreaCode = "617",
                    City = "Brookline",
                    Country = "US",
                    Latitude = 1L,
                    Longitude = 2L,
                    PostalCode = "02445",
                    State = "MA",
                    TimeZone = "EST"
                };

            Assert.AreEqual<int>(0, zip.Id, "Id should be zero before save.");

            Context.GetTable(typeof(ZipCodeInfo)).InsertOnSubmit(zip);
            Context.SubmitChanges();

            Assert.AreNotEqual<int>(0, zip.Id, "Id should not be zero after save.");

            using (DataContext reloadingContext = CreateDataContext())
            {
                ZipCodeInfo reloadedZip = reloadingContext.GetTable<ZipCodeInfo>().FirstOrDefault();

                Assert.IsNotNull(reloadedZip, "Could not find the new zip.");

                Assert.AreEqual<int>(zip.Id, reloadedZip.Id, "Id didn't match.");

                Assert.AreEqual<long>(zip.Longitude, reloadedZip.Longitude, "Reloaded longitude didn't match.");
                Assert.AreEqual<long>(zip.Latitude, reloadedZip.Latitude, "Reloaded latitude didn't match.");
            }
        }
    }
}
